/*
    sound.c - sound core routines
    Copyright (C) 2007  Ch. Klippel <ck@mamalala.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/


/* =================================== Includes ============================ */

#include "irq.h"
#include "sound.h"
#include "soundirq.h"
#include "lpc2220.h"


/* ================== Datentypen und Variablendefinitionen ================= */

#define AUTO_OFF_TIME    0x4000

extern unsigned int              tval;
extern unsigned char             bl_val, cmp_val;
extern unsigned int              slen;
extern unsigned char             *sdata;
extern volatile unsigned char    sact;
extern char                      out1;
extern char                      last_sample;

unsigned char                    timeout;
unsigned int                     auto_timeout;


/* ======================= Funktionen und Programmcode ===================== */

/*
   ----------------------------------------------------------------------------
   Funktion: startSoundIRQ
   ----------------------------------------------------------------------------
*/
void startSoundIRQ (void)
{

   /* ----- Programmcode ----- */
    timeout = 0;
    auto_timeout = 0;
    out1 = 0;

    PWMTC = 0;
//    PWMPR = 0;
    PWMPR = 7;
    PWMMR0 = 0x1E6;    // pwm rate
    PWMMR2 = 0x00;    // pwm value
    PWMLER = 0x05;
    PWMMCR = 0x03;
    PWMPCR = (1<<10);
    PWMTCR = 0x03;
    PWMTCR = 0x09;

    //VICVectAddr0 = (unsigned long)& (soundIRQ);
    //VICVectCntl0 = VIC_SLOT_EN | INT_SRC_PWM;
    VICIntSelect |= INT_PWM;
    VICIntEnable = INT_PWM;
}
/* ----- Ende startSoundIRQ ------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: initSound
   ----------------------------------------------------------------------------
*/
void initSound (void)
{

   /* ----- Programmcode ----- */
    slen = 0;
    sdata = 0;
    sact = 0;
    tval = 0;
    last_sample = 0;
    bl_val = 0x3F;
}
/* ----- Ende initSound ---------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: switchSound
   ----------------------------------------------------------------------------
*/
void switchSound (unsigned char onoff)
{

   /* ----- Programmcode ----- */
    if (onoff)
    {
        FIOSET0 |= (1<<11);
    }
    else
    {
        FIOCLR0 |= (1<<11);
    }
}
/* ----- Ende switchSound -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: playSound
   ----------------------------------------------------------------------------
*/
void playSound (unsigned char *sound, unsigned long len)
{

   /* ----- Programmcode ----- */
    PWMPR = 0;
    PWMPC = 0;
    sdata = sound;
    slen = len;
    tval = 0;
    sact = 1;
    switchSound (SOUND_ON);
}
/* ----- Ende playSound ---------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: stopSound
   ----------------------------------------------------------------------------
*/
void stopSound (void)
{

   /* ----- Programmcode ----- */
    switchSound (SOUND_OFF);
    PWMPR = 7;
    sact = 0;
}
/* ----- Ende stopSound ---------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: waitSound
   ----------------------------------------------------------------------------
*/
void waitSound (void)
{

   /* ----- Programmcode ----- */
    while (sact) { }
}
/* ----- Ende waitSound ---------------------------------------------------- */

