/*
    buffer.h - functions for buffers
    Copyright (C) 2008 Telekatz <telekatz@gmx.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/


/* =================================== Includes ============================ */


/* ================== Datentypen und Variablendefinitionen ================= */


#ifndef BUFFER_H
#define BUFFER_H

typedef struct cBuffer_
{
    unsigned char* data;
    unsigned short size;
    unsigned short len;
    unsigned short index;
} cBuffer_t, *cBuffer_p;


/* ======================== Prototyping ==================================== */


/* Initialisiert den Buffer mit Zeiger auf Buffer-Strukt, Pointer auf Speicherbereich und Groesse des Buffers */
void cBufferInit (struct cBuffer_ *buffer, unsigned char *start, unsigned short size);

/* Setzt den Buffer mit Laenge und Index zurueck auf NULL */
void cBufferFlush (struct cBuffer_ *buffer);

/* Fuegt dem Buffer ein Zeichen hinzu */
unsigned char cBufferAdd (struct cBuffer_ *buffer, unsigned char data);

/* Fuegt dem Buffer den Inhalt eines 2. Buffers mit der Laenge len hinzu */
unsigned short cBufferAddBuffer (struct cBuffer_ *buffer, const void* data, unsigned short len);

/* Return one byte from the buffer an set index and length appropriately */
unsigned char cBufferGet (struct cBuffer_ *buffer);

/* Return len bytes from the buffer an set index and length appropriately */
unsigned short cBufferGetBuffer (struct cBuffer_ *buffer, unsigned char* data, unsigned short len);

/* Return len bytes from the buffer but keep the index and length of the buffer untouched  */
unsigned short cBufferLookBuffer (cBuffer_p pBuffer, unsigned char* data, unsigned short pos, unsigned short len);

/** The return value indicates the relationship between the buffers.
 *
 * Relationship of first count bytes of pBuffer and buf2
 * < 0 buffer less than buf2
 * 0 buffer identical to buf2
 * > 0 buffer greater than buf2
 */
int cBufferCmp (cBuffer_p pBuffer, const void* buf2, int count);

/** Find a substring.
 * The function behaves like @see strstr
 *
 * @param pos start position for search in buffer
 * @param count of by bytes to be searched
 *
 * @return the position of the first occurrence
 *         -1 if the buf2 has not been found in buffer
 */
int cBufferStr (cBuffer_p pBuffer, const void* buf2, unsigned short pos, unsigned short len);


#endif


/* =================================== Ende ================================ */


