/*
   global.c
   Copyright (C) 2007  Colibri <colibri_dvb@lycos.com>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/


/* =================================== Includes ============================ */


#include "lpc2220.h"
#include "rf.h"
#include "global.h"


/* ================== Datentypen und Variablendefinitionen ================= */

#define debugbuffersize   50
unsigned char debugoutbuf [debugbuffersize];


/* ======================= Funktionen und Programmcode ===================== */


/*
   ----------------------------------------------------------------------------
   Funktion: memcpy
   ----------------------------------------------------------------------------
*/
void *memcpy (void *dest,void *src,int count)
{
   /* ----- lokale Variablen ----- */
   char *tmp = (char *) dest, *s = (char *) src;


   /* ----- Programmcode ----- */
   while (count--)
      *tmp++ = *s++;

   return dest;
}
/* ----- Ende memcpy ------------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: memcmp
   ----------------------------------------------------------------------------
*/
int memcmp (const void * cs,const void * ct,int count)
{
   /* ----- lokale Variablen ----- */
   const unsigned char *su1, *su2;
   signed char res = 0;


   /* ----- Programmcode ----- */
   for ( su1 = cs, su2 = ct; 0 < count; ++su1, ++su2, count--)
      if ((res = *su1 - *su2) != 0)
         break;
      return res;
}
/* ----- Ende memcmp ------------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: GetHWversion
   ----------------------------------------------------------------------------
*/
unsigned char GetHWversion (void)
{

   /* ----- Programmcode ----- */
   if (( (IOPIN2 >> 28) & 3) == 3)
   {
      return (( (( (IOPIN0 >> 5) & 1) << 4) | (( (IOPIN2 >> 30) & 1) << 3) | (( (IOPIN2 >> 16) & 1) << 2) | 3));
   }
   return (( (IOPIN2 >> 27) & 2) + ((IOPIN2 >> 29) & 1) + (( (IOPIN1 >> 23) & 7) << 2));
}
/* ----- Ende GetHWversion ------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: puts
   ----------------------------------------------------------------------------
*/
int puts (const char* string)
{
   /* ----- lokale Variablen ----- */
   struct RFendpoint_* cur_ep;
   volatile unsigned char i;


   /* ----- Programmcode ----- */
   cur_ep = openEP (debugoutbuf,0, RF_PACKET_TEST);

   if (cur_ep)
   {
      for (i=0; (i<debugbuffersize) && string[i];i++)
      {
         cur_ep->buffer[i] = string[i];
      }

      cur_ep->buffer[i] = 0;
      cur_ep->bufferlen = i+1;
      cur_ep->flags |= EPenabled | EPoutput | EPnewdata;


      RF_changestate (RFtx);
      // wait until all date are sent
      while (cur_ep->flags & EPnewdata);
      closeEP (cur_ep);
   }
   return 1;
}
/* ----- Ende puts --------------------------------------------------------- */


/* =================================== Ende ================================ */

