/*
menu.c - menuing system
Copyright (C) 2007  Ch. Klippel <ck@mamalala.net>

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/

/* =================================== Includes ============================ */


#include "global.h"
#include "menu.h"
#include "lcd.h"
#include "fonty.h"
#include "keyboard.h"
#include "lpc2220.h"


/* ================== Datentypen und Variablendefinitionen ================= */

#if FF_LARGE_MENU
#define MENU_FONT          SMOOTHFONT
#define MENU_LINE_HEIGHT   (SMOOTHFONT_HEIGHT+2)
#else
#define MENU_FONT          SMALLFONT
#define MENU_LINE_HEIGHT   SMALLFONT_HEIGHT+1
#endif

#define MENU_HEAD_LINE_HEIGHT     (BOLDFONT_HEIGHT+2+2+2)
#define MENU_LINE_WIDTH           (LCD_SIZE_X - 10)
#define MENU_BOTTOM_LINE_HEIGHT   (SMALLFONT_HEIGHT+5)

#define MENU_ARROW_SIZE  (min(MENU_LINE_HEIGHT-6, (LCD_SIZE_X-MENU_LINE_WIDTH-1)/2+1))

#define MENU_START_Y   MENU_HEAD_LINE_HEIGHT
#define MENU_LIST_LEN  ((LCD_SIZE_Y - MENU_HEAD_LINE_HEIGHT - MENU_BOTTOM_LINE_HEIGHT) / MENU_LINE_HEIGHT)
//-----------------------------------------------------------------------------


/* ======================= Funktionen und Programmcode ===================== */

/*
   ----------------------------------------------------------------------------
   Funktion: menu_execentry
   ----------------------------------------------------------------------------
*/
int menu_execentry (struct MENU_ENTRY *entry)
{
   /* ----- lokale Variablen ----- */
   unsigned int *val;
   struct MENU *m;


   /* ----- Programmcode ----- */
   draw_block (0, 100, LCD_SIZE_X, 8, LCD_COLOR_B, DRAW_ERASE);
   switch (entry->type)
   {
      case INTENTRY:
         val = (unsigned int*)entry->data;
         draw_block (0, 50, LCD_SIZE_X, 8, LCD_COLOR_B, DRAW_ERASE);
         draw_hexW  (0, 50, *val, LCD_COLOR_B, DRAW_PUT);
      break;
      case FUNCENTRY:
         entry->exec (entry->data);
         return 1;
      break;
      case MENUENTRY:
         m = (MENU*)entry->data;
         if (m->num_entries != 0)
         {
            menu_exec ((MENU*)entry->data);
            return 1;
         }
      break;
      default:
      break;
   }
   return 0;
}   // menu_execentry
/* ----- Ende menu_execentry ----------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: menu_exec
   ----------------------------------------------------------------------------
*/
void menu_exec (struct MENU *menu)
{
   /* ----- lokale Variablen ----- */
   unsigned char csel, set, total;
   unsigned char moff;
   unsigned char moffold;


   /* ----- Programmcode ----- */
   csel = 0;
   moff = 0;

again:
   lcd_fill (0);

   //-----------------------------------------
   // Menu head line
   set_font (BOLDFONT);
   draw_string (0,  0, (char*)menu->name, LCD_COLOR_B, DRAW_PUT);
   draw_block  (0, BOLDFONT_HEIGHT+2, LCD_SIZE_X, 2, LCD_COLOR_B, DRAW_PUT);
   // Menu bottom line
   draw_block  (0, LCD_SIZE_Y-SMALLFONT_HEIGHT-3, LCD_SIZE_X, 2, LCD_COLOR_B, DRAW_PUT);
   //-----------------------------------------

   total = menu->num_entries;
   moffold = total;

   while (KEY_Exit) {};

   while ( !KEY_Exit )
   {
      if (moff!=moffold)
      {
         set_font (MENU_FONT);
         draw_block (0, MENU_START_Y, LCD_SIZE_X, MENU_LINE_HEIGHT*MENU_LIST_LEN, LCD_COLOR_B, DRAW_ERASE);
         for (set=moff; (set<total) && (set-moff<MENU_LIST_LEN);set++)
         {
            draw_string (1, MENU_START_Y+1 + ((set-moff) * MENU_LINE_HEIGHT),
                         (char*)menu->entry [set].name, LCD_COLOR_B, DRAW_PUT);
         }
         if (total > moff+MENU_LIST_LEN)
         {
            draw_arrow (MENU_LINE_WIDTH+1, MENU_LINE_HEIGHT*MENU_LIST_LEN+3,
                        ARROW_DOWN, MENU_ARROW_SIZE, LCD_COLOR_B, DRAW_PUT);
         }
         if (moff)
         {
            draw_arrow (MENU_LINE_WIDTH+1, MENU_START_Y+3,
                        ARROW_UP, MENU_ARROW_SIZE, LCD_COLOR_B, DRAW_PUT);
         }
         draw_block (0, MENU_START_Y + ((csel-moff) * MENU_LINE_HEIGHT),
                     MENU_LINE_WIDTH, MENU_LINE_HEIGHT, LCD_COLOR_B, DRAW_XOR);
      }
      moffold = moff;

      //-----------------------------------------
      // Menu bottom line
      set_font (SMALLFONT);
      draw_block  ( 0, LCD_SIZE_Y-SMALLFONT_HEIGHT, LCD_SIZE_X, SMALLFONT_HEIGHT, LCD_COLOR_B, DRAW_ERASE);
      draw_string (10, LCD_SIZE_Y-SMALLFONT_HEIGHT, (char*)menu->entry [csel].help, LCD_COLOR_B, DRAW_PUT);
      if (csel < 9)
         draw_char (0, LCD_SIZE_Y-SMALLFONT_HEIGHT, '1' + csel, LCD_COLOR_B, DRAW_PUT);
      if (csel == 9)
         draw_char (0, LCD_SIZE_Y-SMALLFONT_HEIGHT, '0', LCD_COLOR_B, DRAW_PUT);
      //-----------------------------------------

      waitKeyUpDown ();
      draw_block (0, MENU_START_Y + ((csel-moff) * MENU_LINE_HEIGHT),
                  MENU_LINE_WIDTH, MENU_LINE_HEIGHT, LCD_COLOR_B, DRAW_XOR);
      if (KEY_OK)
      {
         if (menu_execentry ((MENU_ENTRY*)&menu->entry[csel]))
            goto again;
      }

      if (KEY_Up)
      {
         csel--;
         if (csel<moff)
            moff--;
         if (csel >= total)
         {
            if (total > MENU_LIST_LEN)
               moff = total - MENU_LIST_LEN;
            csel = total-1;
         }
      }
      if (KEY_Down)
      {
         csel++;
         if (csel == MENU_LIST_LEN+moff)
            moff++;
         if (csel == total)
         {
            moff = 0;
            csel = 0;
         }
      }
      if (KEY_Left)
      {
         csel = 0;
         moff = 0;
      }
      if (KEY_Right)
      {
         csel = total -1;
         if (total > MENU_LIST_LEN)
            moff = total - MENU_LIST_LEN;
      }

      {
         unsigned char num = getKeynum ();
         if (num >= 16 && num <= 25)
         {
            csel = num - 16;
            if (csel >= total)
               csel = total-1;
            if (csel < moff)
               moff = csel;
            if (csel >= MENU_LIST_LEN-moff)
               moff = csel - MENU_LIST_LEN+1;
            if (menu_execentry ((MENU_ENTRY*)&menu->entry[csel]))
               goto again;
        }

      }
      draw_block (0, MENU_START_Y + ((csel-moff) * MENU_LINE_HEIGHT),
                  MENU_LINE_WIDTH, MENU_LINE_HEIGHT, LCD_COLOR_B, DRAW_XOR);
   }   // while (!KEY_Exit)
}   // menu_exec
/* ----- Ende menu_exec ---------------------------------------------------- */



/*
   ----------------------------------------------------------------------------
   Funktion: init_menu
   ----------------------------------------------------------------------------
*/
void init_menu (void)
{
   /* ----- lokale Variablen ----- */

   /* ----- Programmcode ----- */

}
/* ----- Ende init_menu ---------------------------------------------------- */


/* =================================== Ende ================================ */

