/*
    settingsmenue.c
    Copyright (C) 2008  <telekatz@gmx.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/

/* =================================== Includes ============================ */


/* ================== Datentypen und Variablendefinitionen ================= */


#include "menu.h"
#include "settingsmenu.h"
#include "lcd.h"
#include "fonty.h"
#include "keyboard.h"
#include "rtc.h"
#include "backlight.h"
#include "bfs.h"
#include "rf.h"
#include "controls.h"
#include "global.h"

extern volatile unsigned char bl_val;
extern unsigned int bl_timeout;
extern unsigned char bl_max;

const struct MENU_ENTRY settingsMenuEntries[] =
{
   {change_clock,    0, FUNCENTRY, "Clock",    "set time/date"    },
   {display_setting, 0, FUNCENTRY, "Display",  "Display settings" },
   {RF_setting,      0, FUNCENTRY, "RF",       "RF settings"      }
};

const struct MENU settingsMenu =
{
    NUM_OF_ELEMENTS (settingsMenuEntries), (MENU_ENTRY*)&settingsMenuEntries, "Settings"
};

const struct settings_ default_settings =
{
    0x01,    //version
    1000,    //bl_timeout
    63        //bl_max
};


/* ======================= Funktionen und Programmcode ===================== */

/*
   ----------------------------------------------------------------------------
   Funktion: brightness_onchange
   ----------------------------------------------------------------------------
*/
void brightness_onchange (FORM* form, CONTROL* control)
{
   /* ----- Programmcode ----- */
    bl_max= ((NUMBOX*)control)->value;
    setBacklight (bl_max);
}
/* ----- Ende brightness_onchange ------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: timeout_onchange
   ----------------------------------------------------------------------------
*/
void timeout_onchange (FORM* form, CONTROL* control)
{
   /* ----- Programmcode ----- */
    bl_timeout= ((NUMBOX*)control)->value * 200;
    setBacklight (bl_max);
}
/* ----- Ende timeout_onchange --------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: display_setting
   ----------------------------------------------------------------------------
*/
void display_setting (void* pData)
{

   /* ----- lokale Variablen ----- */
    const LABEL label1        = {BC_Labelnotab,0,15,100,10,BOLDFONT,0,"Brightness",0};
    NUMBOX numbox1            = {BC_Numbox,0,25,13,9,SMALLFONT,bl_val,brightness_onchange,0,0,63};
    const LABEL label2        = {BC_Labelnotab,0,37,100,10,BOLDFONT,0,"Backlight timeout",0};
    NUMBOX numbox2            = {BC_Numbox,0,47,19,9,SMALLFONT,bl_timeout/200,timeout_onchange,0,1,200};

    CONTROL* controls[4] = {    (CONTROL*)&label1,
                                (CONTROL*)&label2,
                                (CONTROL*)&numbox1,
                                (CONTROL*)&numbox2};

    FORM form  = {"Display settings"," ",0,0,0,0,controls,2,4,0,0};


   /* ----- Programmcode ----- */
    form_exec (&form);
}
/* ----- Ende display_setting ----------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: RF_setting
   ----------------------------------------------------------------------------
*/
void RF_setting (void* pData)
{
   /* ----- lokale Variablen ----- */
    struct RFsettings_ RFsettings;
    const LABEL label1        = { BC_Labelnotab, 0, 15, 100, 10, BOLDFONT, 0,"Device address",0};
    NUMBOX numbox1            = { BC_Numbox,     0, 25,  19,  9, SMALLFONT,RFsettings.address,0,0,1,127};
    const LABEL label2        = { BC_Labelnotab, 0, 37, 100, 10, BOLDFONT, 0,"Channel",0};
    NUMBOX numbox2            = { BC_Numbox,     0, 47,  19,  9, SMALLFONT,RFsettings.channel,0,0,0,255};

    CONTROL* controls[4] = {    (CONTROL*)&label1,
                                (CONTROL*)&label2,
                                (CONTROL*)&numbox1,
                                (CONTROL*)&numbox2};

    FORM form  = {"RF settings"," ",0,0,0,0,controls,2,4,0,0};


   /* ----- Programmcode ----- */
    BFS_LoadFile (BFS_ID_RFsettings, sizeof (struct RFsettings_), (unsigned char*) &RFsettings);
    form_exec (&form);

    RFsettings.address = numbox1.value;
    RFsettings.channel = numbox2.value;

    BFS_SaveFile (BFS_ID_RFsettings, sizeof (struct RFsettings_), (unsigned char*) &RFsettings);
    load_RF_setting ();

}
/* ----- Ende RF_setting --------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: load_setting
   ----------------------------------------------------------------------------
*/
void load_setting (void)
{
   /* ----- lokale Variablen ----- */
    struct settings_ settings;


   /* ----- Programmcode ----- */
    if (BFS_LoadFile (BFS_ID_settings, sizeof (struct settings_), (unsigned char*) &settings) != sizeof (struct settings_))
    {
        settings = default_settings;
        BFS_SaveFile (BFS_ID_settings, sizeof (struct settings_), (unsigned char*) &settings);
        draw_string (0, 40, "default settings load", LCD_COLOR_B, DRAW_PUT);
    }
    bl_timeout=settings.bl_timeout;
    bl_max = settings.bl_max;
    setBacklight (bl_max);

}
/* ----- Ende load_setting ------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: enter_settings_menu
   ----------------------------------------------------------------------------
*/
void enter_settings_menu (void* pData)
{
   /* ----- lokale Variablen ----- */
   struct settings_ settings;


   /* ----- Programmcode ----- */
   menu_exec ((struct MENU *)&settingsMenu);

   settings = default_settings;
   settings.bl_timeout=bl_timeout;
   settings.bl_max = bl_max;

   BFS_SaveFile (BFS_ID_settings, sizeof (struct settings_), (unsigned char*) &settings);
}
/* ----- Ende enter_settings_menu ------------------------------------------ */

/* =================================== Ende ================================ */

