/*
    irq.c - irq core functions
    Copyright (C) 2007  Ch. Klippel <ck@mamalala.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/

/* =================================== Includes ============================ */


#include "lpc2220.h"
#include "irq.h"
#include "soundirq.h"
// #include "infraredirq.h"


/* ================== Datentypen und Variablendefinitionen ================= */

#define IRQ_MASK 0x000000C0

extern unsigned char bl_val, cmp_val;


/* ======================= Funktionen und Programmcode ===================== */


/*
   ----------------------------------------------------------------------------
   Funktion: asm_get_cpsr
   ----------------------------------------------------------------------------
*/
static inline unsigned asm_get_cpsr (void)
{
   /* ----- lokale Variablen ----- */
  unsigned long retval;


   /* ----- Programmcode ----- */
  asm volatile (" mrs  %0, cpsr" : "=r" (retval) : /* no inputs */  );
  return retval;
}
/* ----- Ende asm_get_cpsr ------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: asm_set_cpsr
   ----------------------------------------------------------------------------
*/
static inline void asm_set_cpsr (unsigned val)
{
   /* ----- Programmcode ----- */
  asm volatile (" msr  cpsr, %0" : /* no outputs */ : "r" (val)  );
}
/* ----- Ende asm_set_cpsr ------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: enableIRQ
   ----------------------------------------------------------------------------
*/
unsigned enableIRQ (void)
{
   /* ----- lokale Variablen ----- */
  unsigned _cpsr;


   /* ----- Programmcode ----- */
  _cpsr = asm_get_cpsr ();
  asm_set_cpsr (_cpsr & ~IRQ_MASK);
  return _cpsr;
}
/* ----- Ende enableIRQ ---------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: disableIRQ
   ----------------------------------------------------------------------------
*/
unsigned disableIRQ (void)
{
   /* ----- lokale Variablen ----- */
  unsigned _cpsr;


   /* ----- Programmcode ----- */
  _cpsr = asm_get_cpsr ();
  asm_set_cpsr (_cpsr | IRQ_MASK);
  return _cpsr;
}
/* ----- Ende disableIRQ --------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: restoreIRQ
   ----------------------------------------------------------------------------
*/
unsigned restoreIRQ (unsigned oldCPSR)
{
   /* ----- lokale Variablen ----- */
  unsigned _cpsr;


   /* ----- Programmcode ----- */
  _cpsr = asm_get_cpsr ();
  asm_set_cpsr ((_cpsr & ~IRQ_MASK) | (oldCPSR & IRQ_MASK));
  return _cpsr;
}
/* ----- Ende restoreIRQ --------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: FIQ_Routine
   ----------------------------------------------------------------------------
*/
void __attribute__ ((section (".text.fastcode"))) FIQ_Routine (void)
{
   /* ----- Programmcode ----- */
//    while ((PWMIR == 0x01) || (T1IR == 1)) {
    while ( PWMIR == 0x01 ) {
//        if (PWMIR == 0x01)
            soundIRQ ();
//        if (T1IR == 1)
//            irIRQ ();
    }
}
/* ----- Ende FIQ_Routine -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: SWI_Routine
   ----------------------------------------------------------------------------
*/
void SWI_Routine (void)
{
   /* ----- Programmcode ----- */
    while (1) ;
}
/* ----- Ende SWI_Routine -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: UNDEF_Routine
   ----------------------------------------------------------------------------
*/
void UNDEF_Routine (void)
{
   /* ----- Programmcode ----- */
    while (1) ;
}
/* ----- Ende UNDEF_Routine ------------------------------------------------ */



/* =================================== Ende ================================ */

