/*
keyirq.c - keyboard scanning interrupt
Copyright (C) 2007  Ch. Klippel <ck@mamalala.net>

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/


/* =================================== Includes ============================ */

#include "keyboard.h"
#include "keyirq.h"
#include "lpc2220.h"
#include "timerfuncs.h"
#include "global.h"
#include "serial.h"


/* ================== Datentypen und Variablendefinitionen ================= */

volatile unsigned int   keys[2];
static unsigned char    scannerTimeout;
static unsigned char    repeatTimeout;
static unsigned char    fastToggle;
volatile unsigned char  scannerState = 0;

#define S_REPEATDELAY   6
#define S_REPEATRATE    4

#define MASK0 0x18402000
#define MASK3 0x00300000


// defines fuer Debug- und Traceausgaben
#define  DEBUG_KEY      0

#if DEBUG_KEY
   unsigned char  temp_counter = 0;
#endif


/* ======================= Funktionen und Programmcode ===================== */


/* ----------------------------------------------------------------------------
Funktion: GetKeyScannerState

Funktion liefert den internen Status des Key-Scanners zurck

bergabeparameter:      keiner

Rckgabeparameter:      S_IDLE       0x00
                        S_PRESSED    0x01
                        S_HOLD       0x02
                        S_REPEAT     0x03
---------------------------------------------------------------------------- */
unsigned char GetKeyScannerState( void )
{

   /* ----- Programmcode ----- */
   return scannerState;

}
/* ---- Ende GetKeyScannerState -------------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: keyScanner

keyScanner ist die zentrale Statemachine zur Interrupt gesettzten Verarbeitung
von Tastendrcken. Die entsprechenden Tastencodes werden in den globalen
Variablen keys[0] und keys[1] zur Verfgung gestellt.

bergabeparameter:      keiner

Rckgabeparameter:      S_IDLE       0x00
                        S_PRESSED    0x01
                        S_HOLD       0x02
                        S_REPEAT     0x03
---------------------------------------------------------------------------- */
void __attribute__ ((section (".text.fastcode"))) keyScanner (void)
{
   /* ----- lokale Variablen ----- */
   static unsigned int temp;

   keys[0] = 0;
   keys[1] = 0;

   #if DEBUG_KEY
      temp_counter++;

      if ( temp_counter >= 66 )
      {
         temp_counter = 0;
         serial_putc( '*' );
      }
   #endif

   if (ANYKEY)
   {
      FIOMASK0 |= ~MASK0;
      IOSET2= ((1<<19)| (1<<20)| (1<<21)| (1<<22)| (1<<23)| (1<<24));

      keys[0] = FIOPIN0>>1;
      keys[0] |= (IOPIN3 & MASK3) >> 14;

      IOSET2 = (1<<18);
      IOCLR2 = (1<<19);

      keys[0] |= FIOPIN0>> 3;
      keys[0] |= (IOPIN3 & MASK3) >> 16;

      IOSET2 = (1<<19);
      IOCLR2 = (1<<20);

      keys[0] |= FIOPIN0>> 5;
      keys[0] |= (IOPIN3 & MASK3) >> 18;

      IOSET2 = (1<<20);
      IOCLR2 = (1<<21);

      keys[0] |= FIOPIN0 >> 13;
      keys[0] |= (IOPIN3 & MASK3) << 8;

      keys[0] = ~keys[0];
      keys[0] &= 0x3FEAD7FD;

      IOSET2 = (1<<21);
      IOCLR2 = (1<<22);

      keys[1] = FIOPIN0>> 3;
      keys[1] |= (IOPIN3 & MASK3) >> 16;

      IOSET2 = (1<<22);
      IOCLR2 = (1<<23);

      keys[1] |= FIOPIN0>> 5;
      keys[1] |= (IOPIN3 & MASK3) >> 18;

      IOSET2 = (1<<23);
      IOCLR2 = (1<<24);

      keys[1] |= FIOPIN0>> 7;
      keys[1] |= (IOPIN3 & MASK3) >> 20;

      keys[1] = ~keys[1];
      keys[1] &= 0x3FA857F;

      temp = (keys[0] & 0x3FE80000)>>3;
      keys[0] &= 0x0002D7FD;
      keys[0] |= temp;

      temp = (keys[0] & 0x07FFD000)>>1;
      keys[0] &= 0x000007FD;
      keys[0] |= temp;

      temp = keys[0]>>1;
      keys[0] &= 0x00000001;
      keys[0] |= temp;

      temp = (keys[0] & 0x1FFF000)>>1;
      keys[0] &= 0x7FF;
      keys[0] |= temp;

      temp = (keys[1] & 0x03FA8000) >> 8;
      keys[1] &= 0x0000057F;
      keys[1] |= temp;

      temp = keys[0] & 0x00E00000;
      keys[0] &= 0x1FFFFF;
      keys[1] |= (temp>>3);

      FIOMASK0 &= MASK0;
      IOCLR2= ((1<<18)| (1<<19)| (1<<20)| (1<<21)| (1<<22)| (1<<23));
   }

   if ((keys[0] == 0) && (keys[1] == 0))
   {
      scannerTimeout = 0;
      repeatTimeout = S_REPEATDELAY;   // 6
      scannerState = S_IDLE;
   }
   else if (scannerState == S_IDLE)
   {
      scannerState = S_PRESSED;
   }

   switch ( scannerState )
   {
      /*
         #define S_IDLE       0x00
         #define S_PRESSED    0x01
         #define S_HOLD       0x02
         #define S_REPEAT     0x03
      */
      case S_PRESSED:
         scannerTimeout++;

         // SYS_IR wird nur in testmenu.c bei Musik gesetzt und rckgesetzt
         if ((scannerTimeout > repeatTimeout) && (repeatTimeout != 0)  && ! (sysInfo & SYS_IR))
         {
            repeatTimeout = S_REPEATRATE; // 4
            scannerState++;
         }
         break;

      case S_HOLD:
         keys[0] = 0;
         keys[1] = 0;
         scannerState++;
         break;

      case S_REPEAT:
         scannerTimeout++;
         if (scannerTimeout > repeatTimeout)
         {
            keys[0] = 0;
            keys[1] = 0;
            scannerTimeout = 0;
            fastToggle++;
            if ((fastToggle > 4) && (repeatTimeout > 1))
            {
               repeatTimeout >>= 1;
               fastToggle = 0;
            }
         }
         break;
   }   // switch (scannerState)
}
/* ---- Ende keyScanner ---------------------------------------------------- */

/* =================================== Ende ================================ */

