/*
    rtc.h -
    Copyright (C) 2008  <telekatz@gmx.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/

#ifndef RTC_H
#define RTC_H


/* =================================== Includes ============================ */

#include "i2c.h"


/* ================== Datentypen und Variablendefinitionen ================= */

#define RTC_LARGE_DIGIT_HEIGHT   45
#define RTCslaveaddr   0xA2

struct time_
{
   unsigned char second;
   unsigned char minute;
   unsigned char hour;
   unsigned char tag;
};
#define offset_time   0x02

struct date_
{
   unsigned char day;
   unsigned char weekday;
   unsigned char month;
   unsigned char year;
};
#define offset_date   0x05

struct rtc_
{
   struct date_ date;
   struct time_ time;
};

extern volatile struct rtc_ rtc;

extern volatile unsigned char timeInfo;
#define timechanged     0x01
#define setnewtime      0x02
#define time_accurate   0x04

extern const unsigned char days[14];


/* ======================== Prototyping ==================================== */


void initRTC      (void);
void getTimeDate  (struct time_* t, struct date_ *d);
void setTimeDate  (struct time_* t, struct date_ *d);
void printTime    (unsigned char x, unsigned char y, struct time_* t, unsigned char format);
void printDate    (unsigned char x, unsigned char y, struct date_* d, unsigned char format);
void rtcIRQ       (void);
void startRtcIRQ  (void);
void change_clock (void* pData);
void calcweekday  (struct date_* d);

/// Send the time via RF packet @see RF_PACKET_TIME to dest
void send_time (unsigned char dest, unsigned char wor);

/// Request to receive time via RF packet @see RF_PACKET_TIME from dest
void ask_for_time (unsigned char dest);

void calcBCD (unsigned char* value, char a, char max, char min);
unsigned char checkBCD (unsigned char value, char max, char min);

#endif

/* =================================== Ende ================================ */

