/**********************************************************************
@file       starligt.h

@brief      Beinhaltet kompletten Projekt-Code fr StarLight

-----------------------------------------------------------------------
@author  2010 Markus Fulde
-----------------------------------------------------------------------
Following compile flags are usable:


-----------------------------------------------------------------------
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-----------------------------------------------------------------------
@History

1.00 17092010    MaFu    Ersterstellung

@end
***********************************************************************/
#ifndef __STARLIGHT_H__
#define __STARLIGHT_H__


/* =============================== Includes ================================ */

#include "buffer.h"


/* ================== Datentypen und Variablendefinitionen ================= */

// Timeout fr Datenempfang
#define STARLIGHT_DATA_TIMEOUT      TIME_MSEC2TICKS (3000)  // Makro rechnet auf m ms Tic um!!

// Allgemeine Definitionen fr Anzahl Dimmer
#define STARLIGHT_DIMMER_1                      1
#define STARLIGHT_DIMMER_2                      2
#define STARLIGHT_DIMMER_3                      3
#define STARLIGHT_DIMMER_4                      4
#define STARLIGHT_DIMMER_5                      5
#define STARLIGHT_DIMMER_6                      6
#define STARLIGHT_DIMMER_7                      7
#define STARLIGHT_DIMMER_8                      8
#define STARLIGHT_DIMMER_ANZAHL                 8


// Allgemeine Arbeitskonstanten
#define STARLIGHT_FUNKTION_EIN                  0x01
#define STARLIGHT_FUNKTION_AUS                  0x00

// Version der Datendefinition im Flash
#define STARLIGHT_AKT_DATA_DEFINITION           0x02

// Dimmwerte
#define STARLIGHT_DIMMER_DATA_MIN               1   // Minimaler Dimmerwert
#define STARLIGHT_DIMMER_DATA_MAX               31  // Maximaler Dimmerwert

#define STARLIGHT_DIMMER_33                     10  // Dimmwert fr 33%
#define STARLIGHT_DIMMER_66                     20  // Dimmwert fr 66%
#define STARLIGHT_DIMMER_100                    31  // Dimmwert fr 100%


// Koordinaten fr Anzeige auf Display
#define STARLIGHT_DIMMER_POS_X                  1
#define STARLIGHT_DIMMER_POS_Y                  1

#define STARLIGHT_BALKEN_POS_X                  15
#define STARLIGHT_BALKEN_POS_Y                  35
#define STARLIGHT_BALKEN_POS_BREITE             30
#define STARLIGHT_BALKEN_POS_HOEHE              115
#define STARLIGHT_BALKEN_RAHMENBREITE           2
#define STARLIGHT_BALKEN_ABSTAND                4
#define STARLIGHT_BALKEN_TEXT_DELTA_X           3
#define STARLIGHT_BALKEN_TEXT_DELTA_Y           22

#define STARLIGHT_BESCHRIFTUNG_LINKS_POS_X      1
#define STARLIGHT_BESCHRIFTUNG_LINKS_DELTA_X    6
#define STARLIGHT_BESCHRIFTUNG_LINKS_DELTA_Y    8

#define STARLIGHT_BESCHRIFTUNG_RECHTS_POS_X     48

#define STARLIGHT_BESCHRIFTUNG_TEXT_POS_X       60
#define STARLIGHT_BESCHRIFTUNG_TEXT_DIMM_POS_Y  60
#define STARLIGHT_BESCHRIFTUNG_TEXT_STAT_POS_Y  90
#define STARLIGHT_BESCHRIFTUNG_TEXT_TEMP_POS_Y  120
#define STARLIGHT_BESCHRIFTUNG_TEXT_DELTA       10

// Mensteuerung
#define STARLIGHT_SHORTCUT_KEY_A                0x00
#define STARLIGHT_SHORTCUT_KEY_B                0x01
#define STARLIGHT_SHORTCUT_KEY_C                0x02
#define STARLIGHT_SHORTCUT_KEY_D                0x03

// Definitionen fr Dimmer 1 - Esstisch
#define  STARLIGHT_GUI_DIMMER1_X                31
#define  STARLIGHT_GUI_DIMMER1_Y                125

// Definitionen fr Dimmer 2 - Beleuchtungsreihe Fenster
#define  STARLIGHT_GUI_DIMMER2_X                20
#define  STARLIGHT_GUI_DIMMER2_Y1               30
#define  STARLIGHT_GUI_DIMMER2_Y2               71
#define  STARLIGHT_GUI_DIMMER2_Y3               107
#define  STARLIGHT_GUI_DIMMER2_Y4               143

// Definitionen fr Dimmer 3 - Sternenhimmel
#define  STARLIGHT_GUI_DIMMER3_X                51
#define  STARLIGHT_GUI_DIMMER3_Y                73
#define  STARLIGHT_GUI_DIMMER3_W                26
#define  STARLIGHT_GUI_DIMMER3_H                26
#define  STARLIGHT_GUI_DIMMER3_SON              3
#define  STARLIGHT_GUI_DIMMER3_SOFF             1

// Definitionen fr Dimmer 4 - Durchgangsbeleuchtung
#define  STARLIGHT_GUI_DIMMER4_X                64
#define  STARLIGHT_GUI_DIMMER4_Y1               43
#define  STARLIGHT_GUI_DIMMER4_Y2               130

// Definitionen fr Dimmer 5 - Bilder
#define  STARLIGHT_GUI_DIMMER5_X                117
#define  STARLIGHT_GUI_DIMMER5_Y1               43
#define  STARLIGHT_GUI_DIMMER5_Y2               58

// Definitionen fr Dimmer 6 - Wohnzimmertische
#define  STARLIGHT_GUI_DIMMER6_X                88
#define  STARLIGHT_GUI_DIMMER6_Y                53

// Definitionen fr Dimmer 7 - Regalbeleuchtung
#define  STARLIGHT_GUI_DIMMER7_X1               86
#define  STARLIGHT_GUI_DIMMER7_X2               101
#define  STARLIGHT_GUI_DIMMER7_X3               116
#define  STARLIGHT_GUI_DIMMER7_Y                110

// Definitionen fr Dimmer 8 - Beleuchtungsreihe Wand
#define  STARLIGHT_GUI_DIMMER8_X                108
#define  STARLIGHT_GUI_DIMMER8_Y1               30
#define  STARLIGHT_GUI_DIMMER8_Y2               71
#define  STARLIGHT_GUI_DIMMER8_Y3               143

// Definition der Icon-Groessen gerundet zum Rechnen ohne Bruch
#define  STARLIGHT_LAMPE_AUS_H_X                10
#define  STARLIGHT_LAMPE_AUS_H_Y                22

#define  STARLIGHT_LAMPE_AUS_V_X                22
#define  STARLIGHT_LAMPE_AUS_V_Y                10

#define  STARLIGHT_LAMPE_EIN_H_X                18
#define  STARLIGHT_LAMPE_EIN_H_Y                24

#define  STARLIGHT_LAMPE_EIN_V_X                24
#define  STARLIGHT_LAMPE_EIN_V_Y                18

#define  STARLIGHT_LICHT_AUS_X                  10
#define  STARLIGHT_LICHT_AUS_Y                  10

#define  STARLIGHT_LICHT_EIN_X                  18
#define  STARLIGHT_LICHT_EIN_Y                  18

#define  STARLIGHT_BILDERLAMPE_X                14
#define  STARLIGHT_BILDERLAMPE_Y                10

#define  STARLIGHT_GUI_RAHMEN_X                 2
#define  STARLIGHT_GUI_RAHMEN_Y                 15
#define  STARLIGHT_GUI_RAHMEN_W                 126
#define  STARLIGHT_GUI_RAHMEN_H                 143
#define  STARLIGHT_GUI_RAHMEN_S                 2


typedef struct dimmerProgramm_s
{
   unsigned char  dimmerstatus;
   unsigned char  dimmwert;
} dimmerProgramm_t, *dimmerProgramm_p;

typedef struct starlightSettings_s
{
   unsigned char           data_version;
   dimmerProgramm_t        data[4][8];
} starlightSettings_t, *starlightSettings_p;

typedef struct dimmerData_s
{
   unsigned char  status;
   unsigned char  dimmwert;
   unsigned char  temperatur;
   unsigned char  sound;
} dimmerData_t, *dimmerData_p;

typedef struct starlightData_s
{
   /* Flag ob Betty mit Betriebsdaten des Masters Urinitialisiert */
   BOOL                     initialized_flag;

   volatile unsigned long   timer;
   BOOL                     isTimerRunning;

   /* globales Flag ob Sound eingeschaltet oder ausgeschaltet */
   BOOL                     sound_status;

   /* Array fr Betriebsdaten aller 8 Dimmer */
   dimmerData_t             dimmer[8];

} starlightData_t, *starlightData_p;


/* ======================== Prototyping ==================================== */

/* ----------------------------------------------------------------------------
Funktion: starlight_credits

Funktion zur Ausgabe des Titelbildschirms

bergabeparameter:   keiner

Rckgabeparameter:   keiner
---------------------------------------------------------------------------- */
void Starlight_Credits ( void );


/* ----------------------------------------------------------------------------
Funktion: CRC16

Funktione berechnet CRC16 Checksumme fr Kommunikation mit Master. Achtung,
Master-Implementierung erfolgt in AVR-BASCOM. CRC16 Routine ist speziell auf
BASCOM CRC16 abgestimmt!

bergabeparameter:   buffer[] = Zeiger auf Byte-Array
                     length   = Lnge ber die CRC16 gerechnet werden soll

Rckgabeparameter:   CRC16 Checksumme gemBASCOM-Implementierung
---------------------------------------------------------------------------- */
unsigned int CRC16(unsigned char buffer[], unsigned char length);


/* ----------------------------------------------------------------------------
Funktion: FormatedDez

Funktion formatiert eine Dezimalzahl in einen String und fllt vorangestellte
Stellen die nicht benutzt werden mit NULL auf.

bergabeparameter:   string_ptr  = Pointer auf String in den konvertiert wird
                     zahl        = Zahl die konvertiert werden soll
                     stellen     = Stellen die konvertiert werden

Rckgabeparameter:   keiner (es wird auf String-Pointer gearbeitet)
---------------------------------------------------------------------------- */
void FormatedDez ( char *string_ptr, unsigned int zahl, unsigned char stellen );


/* ----------------------------------------------------------------------------
Funktion: StarLight_Init

Funktion initialisiert Daten und Datenstrukturen fr Projekt

bergabeparameter:      keiner

Rckgabeparameter:      TRUE = Funktion war erfolgreich
---------------------------------------------------------------------------- */
BOOL StarLight_Init ( void );


/* ----------------------------------------------------------------------------
Funktion: StarLight_Init_Status

Funktion liefert Initialisierungs-Status der Daten und Datenstrukturen fr Projekt

bergabeparameter:      keiner

Rckgabeparameter:      TRUE  = Initialisierung erfolgt
                        FASLE = Initialisierung nocht nicht erfolgt
---------------------------------------------------------------------------- */
BOOL StarLight_Init_Status( void );


/* ----------------------------------------------------------------------------
Funktion: StarLight_Forced_Reset

Funktion lst ber WD einen SW-Reset aus

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_Forced_Reset ( void );


/* ----------------------------------------------------------------------------
Funktion: StarLight_ShowDimmerData

Funktion gibt alle gespeicherten Dimmer-Daten auf dem Display aus

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_ShowDimmerData ( void );


/* ----------------------------------------------------------------------------
Funktion: StarLight_SoundSchalten

Funktion steuert die Wiedergabe von akustischen Feedback von Master und Slave.
Die Funktion ist als Toggelfunktion umgesetzt.

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_SoundSchalten ( void );


/* ----------------------------------------------------------------------------
Funktion: StarLight_Demomodus

Funktion steuert Demomodus an

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
BOOL StarLight_Demomodus ( void );


/* ----------------------------------------------------------------------------
Funktion: StarLight_AlleLampenSchalten

Funktion steuert alle Lampen auf einmal. Alle Lampen knnen auf einmal EIN bzg.
AUS geschaltet werden

bergabeparameter:      lampen_status = [ FKT_STATUS_AUS / FKT_STATUS_EIN ]

Rckgabeparameter:      TRUE  = Funktion war erfolgreich
                        FALSE = Es ist ein Fehler aufgetreten bzw. Anlage
                                war abgeschaltet.
---------------------------------------------------------------------------- */
BOOL StarLight_AlleLampenSchalten ( unsigned char lampen_status );


/* ----------------------------------------------------------------------------
Funktion: StarLight_LampenDimmen

Mittels dieser Funktion knnen die Dimmer einzeln oder zusammen gesteuert
werden.

bergabeparameter:      dimmer   = [ 1..8 / 0xFF ]
                        dimmwert = [ 1..31 ]

Rckgabeparameter:      TRUE  = Funktion war erfolgreich
                        FALSE = Es ist ein Fehler aufgetreten bzw. Anlage
                                war abgeschaltet.
---------------------------------------------------------------------------- */
BOOL StarLight_LampenDimmen ( unsigned char dimmer, unsigned char dimmwert );


/* ----------------------------------------------------------------------------
Funktion: StarLight_LampenSchalten

Mittels dieser Funktion knnen die Dimmer einzeln oder zusammen geschaltet
werden.

bergabeparameter:      dimmer   = [ 1..8 / 0xFF ]
                        dimmwert = [ 0..1 ]

Rckgabeparameter:      TRUE  = Funktion war erfolgreich
                        FALSE = Es ist ein Fehler aufgetreten bzw. Anlage
                                war abgeschaltet.
---------------------------------------------------------------------------- */
BOOL StarLight_LampenSchalten ( unsigned char dimmer, unsigned char status );


/* ----------------------------------------------------------------------------
Funktion: StarLight_AnlageSchalten

Funktion ist Toggelfunktion und schaltet abhngig vom Zustand System ein und
aus. Beim Wiedereinschalten wird LUM gesetzt.

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_AnlageSchalten ( void );


/* ----------------------------------------------------------------------------
Funktion: StarLight_DialogDimmer

Funktion steuert nach Aufruf der Zifferntasten 1-8 die Dialogfhrung mit
grafischer Benutzerfhrung des Dimmers.

Untersttzte Tasten sind:
   Exit    = Verlassen / Ende
   1-8     = Auswahl eines neuen Dimmers
   VOL+/-  = Dimmstufe verndern
   PROG+/- = anderen Dimmer auswhlen
   ROT     = ausgewhlter Dimmer aus
   GRN    = ausgewhlter Dimmer ein
   MEN    = aufruf des Dimmer-Einstellmens
   A-D     = ShortCuts je nach Programmierung

bergabeparameter:      dimmer = Dimmer der gesteuert werden soll

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_DialogDimmer ( unsigned int dimmerData );



/* ----------------------------------------------------------------------------
Funktion: Starlight_ShortCut_A

Menfunktion fr StarLight Einstellung der Taste A

bergabeparameter:      pData = Universalpointer ( nicht verwendet )

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void Starlight_ShortCut_A (void* pData);


/* ----------------------------------------------------------------------------
Funktion: Starlight_ShortCut_B

Menfunktion fr StarLight Einstellung der Taste A

bergabeparameter:      pData = Universalpointer ( nicht verwendet )

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void Starlight_ShortCut_B (void* pData);


/* ----------------------------------------------------------------------------
Funktion: Starlight_ShortCut_C

Menfunktion fr StarLight Einstellung der Taste A

bergabeparameter:      pData = Universalpointer ( nicht verwendet )

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void Starlight_ShortCut_C (void* pData);


/* ----------------------------------------------------------------------------
Funktion: Starlight_ShortCut_D

Menfunktion fr StarLight Einstellung der Taste A

bergabeparameter:      pData = Universalpointer ( nicht verwendet )

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void Starlight_ShortCut_D (void* pData);


/* ----------------------------------------------------------------------------
Funktion: StarLight_Load_Settings

Funktion initialisiert die Einstellungen fr die Tasten A-D und lies die Daten
aus dem BFS aus.

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_Load_Settings( void );


/* ----------------------------------------------------------------------------
Funktion: StarLight_DimmerSchalten

Mittels dieser Funktion knnen die Dimmer einzeln oder zusammen geschaltet
werden.

bergabeparameter:      dimmer   = [ 1..8 / 0xFF ]
                        status   = [ 0..1 ]
                        dimmwert = [ 1.. 31 ]

Rckgabeparameter:      TRUE  = Funktion war erfolgreich
                        FALSE = Es ist ein Fehler aufgetreten bzw. Anlage
                                war abgeschaltet.
---------------------------------------------------------------------------- */
BOOL StarLight_DimmerSchalten ( unsigned char dimmer, unsigned char status, unsigned char dimmwert );


/* ----------------------------------------------------------------------------
Funktion: StarLight_Anlage_Shortcut

Funktion lst ShortCut-Funktion A aus. Damit werden Dimmer so eingestellt, wie
unter A programmiert.

bergabeparameter:      shortcut = [0..3] = Ziffern 0..3 fr A..D

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_Anlage_Shortcut ( unsigned char shortcut );


/* ----------------------------------------------------------------------------
Funktion: StarLight_DimmerGUI

Funktion bedient ber grafisches Interface die Dimmer. Hierbei knnen die einzel
nen Dimmergruppen 1..8 ber Tasten angewhlt und bedient werden.

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_DimmerGUI ( void );


/* ----------------------------------------------------------------------------
Funktion: StarLight_Check_Uebertemperatur

Funktion prft alle 8 Dimmer auf bertemperatur und gibt Statusbildschirm aus

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_Check_Uebertemperatur ( void );


#endif   // #ifndef __STARLIGHT_H__

/* =================================== Ende ================================ */

