/*
    timerfuncs.c - timer control functions
    Copyright (C) 2007  Ch. Klippel <ck@mamalala.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/


/* =================================== Includes ============================ */


#include "lpc2220.h"
#include "timerfuncs.h"
#include "irq.h"
#include "keyboard.h"


/* ================== Datentypen und Variablendefinitionen ================= */

extern struct CB callbacks[MAX_CB];
extern unsigned long* timeouts[MAX_TO];


/* ======================= Funktionen und Programmcode ===================== */

/*
   ----------------------------------------------------------------------------
   Funktion: startTimerIRQ
   ----------------------------------------------------------------------------
*/
void startTimerIRQ (void)
{
   /* ----- lokale Variablen ----- */
   int i;
   struct CB *cur_cb;


   /* ----- Programmcode ----- */
   for (i=0; i<MAX_CB; i++)
   {
      cur_cb = &callbacks[i];
      cur_cb->state    = SLOT_FREE;
      cur_cb->exec    = 0;
   }

   for (i=0; i<MAX_TO; i++)
      timeouts[i] = 0;

   T0TCR = 0x02;    // reset timer
   T0TC = 1870;
   T0PR = 0x0e;     // 15.000.000 Hz / 15 = 1.000.000 Hz --> PR = 15 - 1 = 0x0e
   T0MR0 = 5000;    //  1.000.000 Hz / 5000 =  200 Hz = 5 msec intervall time
   T0MCR = 0x03;    // reset and issue IRQ on TC == MR0
   T0TCR = 0x01;    // enable timer

   T0overflow = 0;

   VICVectAddr15 = (unsigned long)& (timerIRQ);
   VICVectCntl15 = VIC_SLOT_EN | INT_SRC_TIMER0;
   VICIntEnable = INT_TIMER0;
}
/* ----- Ende startTimerIRQ ------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: wait5ms
   ----------------------------------------------------------------------------
*/
void wait5ms (void) {

   /* ----- lokale Variablen ----- */
   unsigned long x;


   /* ----- Programmcode ----- */
   x=T0TC;
   while (x==T0TC);
   while (x!=T0TC);

}
/* ----- Ende wait5ms ------------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: waitus
   ----------------------------------------------------------------------------
*/
void waitus (unsigned long d) {

   /* ----- lokale Variablen ----- */
   unsigned long x,y;


   /* ----- Programmcode ----- */
   x= (T0TC+d)%5000;
   y= T0overflow;

   if (x<T0TC)
      while (y==T0overflow);
      while (x>T0TC);

}
/* ----- Ende waitus ------------------------------------------------------- */


/**
@brief Alocate a timer, set the time but do *NOT* start.

       Use @see startCB to start the timer

@Param  Timer expiry callback, time intervall

@return index of the allocated timer on success
        0xFF is no timer left
*/

/*
   ----------------------------------------------------------------------------
   Funktion: addTimerCB
   ----------------------------------------------------------------------------
*/
unsigned int addTimerCB (cb_fn cb, unsigned short intervall)
{
   /* ----- lokale Variablen ----- */
   int i;
   struct CB *cur_cb;


   /* ----- Programmcode ----- */
   for (i = 0; i < MAX_CB;)
   {
      cur_cb = &callbacks[i];
      if (cur_cb->state == SLOT_FREE)
      {
         cur_cb->exec          = cb;
         cur_cb->intervall     = intervall;
         cur_cb->cur_intervall = 0;
         cur_cb->state         = SLOT_HALT | SLOT_USED;
         return i;
      }
      i++;
   }
   return 0xFF;
}
/* ----- Ende addTimerCB --------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: removeTimerCB
   ----------------------------------------------------------------------------
*/
unsigned int removeTimerCB (unsigned int cb)
{
   /* ----- Programmcode ----- */
   if (cb < MAX_CB)
   {
      callbacks[cb].state    = SLOT_FREE;
      return MAX_CB;
   }
   return 0xFF;
}
/* ----- Ende removeTimerCB ------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: startCB
   ----------------------------------------------------------------------------
*/
void startCB (unsigned int cb)
{
   /* ----- Programmcode ----- */
   if (cb < MAX_CB)
   {
      callbacks[cb].state &= ~SLOT_HALT;
   }
}
/* ----- Ende startCB ------------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: stopCB
   ----------------------------------------------------------------------------
*/
void stopCB (unsigned int cb)
{
   /* ----- Programmcode ----- */
   if (cb < MAX_CB)
   {
      callbacks[cb].state |= SLOT_HALT;
   }
}
/* ----- Ende stopCB ------------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: 
   ----------------------------------------------------------------------------
*/
void setCBIntervall (unsigned int cb, unsigned short intervall)
{
   /* ----- Programmcode ----- */
   if (cb < MAX_CB)
   {
      callbacks[cb].intervall = intervall;
      callbacks[cb].cur_intervall = 0;
   }
}
/* ----- Ende  ------------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: addTimeout
   ----------------------------------------------------------------------------
*/
unsigned char addTimeout (volatile unsigned long* counter, unsigned long value)
{
   /* ----- lokale Variablen ----- */
   int i;


   /* ----- Programmcode ----- */
   for (i = 0; i < MAX_TO; i++)
   {
      if (timeouts[i] == counter)
      {
         *timeouts[i] = value;
         timeouts[i] = (unsigned long*)counter;
         return (1);
      }
   }

   for (i = 0; i < MAX_TO; i++)
   {
      if (timeouts[i] == 0)
      {
         timeouts[i] = (unsigned long*)counter;
         *timeouts[i] = value;
         return (1);
      }
   }

   *counter = 0;
   return (0);
}   // addTimeout
/* ----- Ende addTimeout --------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: removeTimeout
   ----------------------------------------------------------------------------
*/
unsigned char removeTimeout (volatile unsigned long* counter)
{
   /* ----- lokale Variablen ----- */
   int i;


   /* ----- Programmcode ----- */
   for (i=0; i<MAX_TO; i++)
   {
      if (timeouts[i] == counter)
      {
         timeouts[i] = 0;
         return (1);
      }
   }

   return (0);
}   // removeTimeout
/* ----- Ende removeTimeout ------------------------------------------------ */


/* =================================== Ende ================================ */

